package TrEd::Config;

#
# $Revision: 1.18 $ '
# Time-stamp: <2001-08-13 12:23:13 pajas>
#
# Copyright (c) 2001 by Petr Pajas <pajas@matfyz.cz>
# This software covered by GPL - The General Public Licence
#

BEGIN {
  use Exporter  ();
#  use Tk;			# Tk::strictMotif
  use vars      qw($VERSION @ISA @EXPORT @EXPORT_OK @config_file_search_list);
  @ISA=qw(Exporter);
  $VERSION = "0.1";
  @EXPORT = qw(@config_file_search_list $set_user_config
  $appName
  $buttonsRelief
  $menubarRelief
  $buttonBorderWidth
  $canvasBalloonInitWait
  $activeTextColor
  $treeViewOpts
  $font
  $vLineFont
  $type1font
  $libDir
  $psFontFile
  $psFontAFMFile
  $appIcon
  $sortAttrs
  $psFontName
  $psFontSize
  $macroFile
  $defaultMacroFile
  $prtFmtWidth
  $prtFmtHeight
  $prtVMargin
  $prtHMargin
  $psMedia
  $psFile
  $maximizePrintSize
  $showHidden
  $createMacroMenu
  $maxMenuLines
  $useCzechLocales
  $printColors
  $cstsToFs
  $fsToCsts
  $gzip
  $zcat
  $sgmls
  $sgmlsopts
  $cstsdoctype
  $keyboardDebug
  $hookDebug
  $macroDebug
  $tredDebug
  $defaultTemplateMatchMethod
  $defaultMacroListOrder
  $defCWidth
  $defCHeight
  $geometry
  $maxDisplayedValues
  $maxDisplayedAttributes
  $highlightWindowColor
  $highlightWindowWidth
  $lastAction);
  @EXPORT_OK=qw(&tilde_expand &read_config &set_config &parse_config_line &apply_config &set_default_config_file_search_list);

  use strict;

  @config_file_search_list=();
}

sub set_default_config_file_search_list {
  require FindBin;
  @config_file_search_list=
    ($ENV{HOME}.'/.tredrc',
     (exists $ENV{TREDHOME}) ? $ENV{TREDHOME}.'/tredrc' : (),
     "$FindBin::RealBin/tredrc",
     "$FindBin::RealBin/../lib/tredlib/tredrc",
     "$FindBin::RealBin/tredlib/tredrc",
     "$FindBin::RealBin/../lib/tred/tredrc",
     '/usr/usr/share/config/tredrc');
}

sub tilde_expand {
  my ($a)=@_;
  $a=~s/^\~/$ENV{HOME}/;
  $a=~s/([^\\])\~/$1$ENV{HOME}/g;
  return $a;
}

sub parse_config_line {
  local $_=shift;
  my $confs=shift;
  my $key;
  unless (/^[;\#]/ or /^$/) {
    chomp;
    if (/^\s*([a-zA-Z_]+[a-zA-Z_0-9]*)\s*=\s*('(?:[^\\']|\\.)*'|"(?:[^\\"]|\\.)*"|(?:\s*(?:[^;\\\s]|\\.)+)*)/) {
      $key=lc($1);
      $confs->{$key}=$2;
      $confs->{$key}=~s/\\(.)/$1/g;
      $confs->{$key}=$1 if ($confs->{$key}=~/^'(.*)'$/ or $confs->{$key}=~/^"(.*)"$/);
    }
  }
}

sub read_config {
  #
  # Simple configuration file handling
  #
  my %confs;
  my ($key, $f);
  local *F;
  my $openOk=0;
  my $config_file;

  foreach $f (@_,@config_file_search_list) {
    if (defined($f) and open(F,"<$f")) {
      print STDERR "Using resource file $f\n";
      while (<F>) {
	parse_config_line($_,\%confs);
      }
      close F;
      $openOk=1;
      $config_file=$f;
      last;
    }
  }
  unless ($openOk) {
    print STDERR
      "Warning: Cannot open any file in:\n",
      join(":",@config_file_search_list),"\n" .
      "         Using configuration defaults!\n";
  }
  set_config(\%confs);
  return $config_file;
}

sub apply_config {
  my %confs;
  foreach (@_) {
    parse_config_line($_,\%confs);
  }
  set_config(\%confs);
}

sub val_or_def {
  my ($confs,$key,$def)=@_;
  return ((exists $confs->{$key}) ? $confs->{$key} : $def);
}

sub set_config {
  my ($confs)=@_;

  $appName=val_or_def($confs,"appname","TrEd ver. 0.5");
  $buttonsRelief=val_or_def($confs,"buttonsrelief",'flat');
  $menubarRelief=val_or_def($confs,"menubarrelief",'flat');
  $buttonBorderWidth=val_or_def($confs,"buttonsborder",2);
  $canvasBalloonInitWait=val_or_def($confs,"hintwait",1000);
  $activeTextColor=val_or_def($confs,"activetextcolor",'blue');

  $highlightWindowColor=val_or_def($confs,"highlightwindowcolor",'black');
  $highlightWindowWidth=val_or_def($confs,"highlightwindowwidth",3);

  $treeViewOpts->{baseXPos}	      =	 val_or_def($confs,"basexpos",15);
  $treeViewOpts->{baseYPos}	      =	 val_or_def($confs,"baseypos",15);
  $treeViewOpts->{nodeWidth}	      =	 val_or_def($confs,"nodewidth",7);
  $treeViewOpts->{nodeHeight}	      =	 val_or_def($confs,"nodeheight",7);
  $treeViewOpts->{useAdditionalEdgeLabelSkip}
                                      =
					 val_or_def($confs,"useadditionaledgelabelskip",1);
  $treeViewOpts->{currentNodeWidth}   =	 val_or_def($confs,"currentnodewidth",$nodeWidth);
  $treeViewOpts->{currentNodeHeight}  =	 val_or_def($confs,"currentnodeheight",$nodeHeight);
  $treeViewOpts->{nodeXSkip}	      =	 val_or_def($confs,"nodexskip",10);
  $treeViewOpts->{nodeYSkip}	      =	 val_or_def($confs,"nodeyskip",10);
  $treeViewOpts->{edgeLabelSkipAbove} =	 val_or_def($confs,"edgelabelskipabove",10);
  $treeViewOpts->{edgeLabelSkipBelow} =	 val_or_def($confs,"edgelabelskipbelow",10);
  $treeViewOpts->{xmargin}	      =	 val_or_def($confs,"xmargin",2);
  $treeViewOpts->{ymargin}	      =	 val_or_def($confs,"ymargin",2);
  $treeViewOpts->{lineWidth}	      =	 val_or_def($confs,"linewidth",1);
  $treeViewOpts->{lineColor}	      =	 val_or_def($confs,"linecolor",'black');
  $treeViewOpts->{lineArrow}	      =	 val_or_def($confs,"linearrow",'none');
  $treeViewOpts->{nodeColor}	      =	 val_or_def($confs,"nodecolor",'yellow');
  $TrEd::Print::bwModeNodeColor	      =	 val_or_def($confs,"bwprintnodecolor",'white');
  $treeViewOpts->{nodeOutlineColor}   =	 val_or_def($confs,"nodeoutlinecolor",'black');
  $treeViewOpts->{hiddenNodeColor}    =	 val_or_def($confs,"hiddennodecolor",'black');
  # $activeNodeColor		      =	 val_or_def($confs,"activenodecolor",'blue');
  $treeViewOpts->{currentNodeColor}   =	 val_or_def($confs,"currentnodecolor",'red');
  $treeViewOpts->{nearestNodeColor}   =	 val_or_def($confs,"nearestnodecolor",'green');
  $treeViewOpts->{textColor}	      =	 val_or_def($confs,"textcolor",'black');
  $treeViewOpts->{textColorShadow}    =	 val_or_def($confs,"textcolorshadow",'darkgrey');
  $treeViewOpts->{textColorHilite}    =	 val_or_def($confs,"textcolorhilite",'darkgreen');
  $treeViewOpts->{textColorXHilite}   =	 val_or_def($confs,"textcolorxhilite",'darkred');

  foreach (0..9) {
    $treeViewOpts->{customColors}->[$_]=$confs->{"customcolor$_"} if $confs->{"customcolor$_"};
  }

  $treeViewOpts->{boxColor}	       = val_or_def($confs,"boxcolor",'LightYellow');
  $treeViewOpts->{currentBoxColor}     = val_or_def($confs,"currentboxcolor",'yellow');
  $treeViewOpts->{hiddenBoxColor}      = val_or_def($confs,"hiddenboxcolor",'gray');

  $treeViewOpts->{edgeBoxColor}	       = val_or_def($confs,"edgeboxcolor",'#fff0e0');
  $treeViewOpts->{currentEdgeBoxColor} = val_or_def($confs,"edgecurrentboxcolor",'#ffe68c');
  $treeViewOpts->{hiddenEdgeBoxColor}  = val_or_def($confs,"edgehiddenboxcolor","DarkGrey");

  $treeViewOpts->{clearTextBackground} = val_or_def($confs,"cleartextbackground",1);

  $treeViewOpts->{backgroundColor}     = val_or_def($confs,"backgroundcolor",undef);
  $treeViewOpts->{noColor}	       = val_or_def($confs,"allowcustomcolors",0);
  $treeViewOpts->{drawBoxes}	       = val_or_def($confs,"drawboxes",0);
  $treeViewOpts->{drawEdgeBoxes}       = val_or_def($confs,"drawedgeboxes",0);
  $treeViewOpts->{highlightAttributes} = val_or_def($confs,"highlightattributes",1);
  $treeViewOpts->{showHidden} = val_or_def($confs,"showhiddne",0);;

  $font=(exists $confs->{font}) ? $confs->{font} :
    (($^O=~/^MS/) ? 'family:Helvetica,size:10' : '-*-helvetica-medium-r-normal-*-12-*-*-*-*-*-iso8859-2');
  $treeViewOpts->{font}=$font;
  $vLineFont=val_or_def($confs,"vlinefont",$font);
  $type1font=(exists $confs->{type1font}) ? $confs->{type1font} :
    (($^O=~/^MS/) ? $font : '-ult1mo-arial-medium-r-*-*-*-*-*-*-*-*-iso8859-2');

  $libDir=tilde_expand($confs->{libdir})
    if (exists $confs->{libdir});
  unshift @INC,$libDir unless (grep($_ eq $libDir, @INC));
  if (exists $confs->{psfontfile}) {
    $psFontFile=tilde_expand($confs->{psfontfile});
    $psFontFile="$libDir/".$psFontFile if (not -f $psFontFile and -f 
					   "$libDir/".$psFontFile);
  } else {
    $psFontFile="$libDir/fonts/ariam___.pfa";
  }
  if (exists $confs->{psfontafmfile}) {
    $psFontAFMFile=$confs->{psfontafmfile};
  } else {
    $psFontAFMFile=$psFontFile;
    $psFontAFMFile=~s/\.[^.]+$/.afm/;
    unless (-f $psFontAFMFile) {
      $psFontAFMFile=~s!/([^/]+)$!/afm/$1!;
    }
  }

  $appIcon=(exists $confs->{appicon}) ? tilde_expand($confs->{appicon}) : "$libDir/tred.xpm";
  $macroFile=tilde_expand($confs->{macrofile}) if (exists $confs->{macrofile});
  $defaultMacroFile=(exists $confs->{defaultmacrofile}) ? tilde_expand($confs->{defaultmacrofile}) : "$libDir/tred.def";

  $sortAttrs	     =	val_or_def($confs,"sortattributes",1);
  $psFontName	     =	val_or_def($confs,"psfontname","Arial-Medium");
  $psFontSize	     =	val_or_def($confs,"psfontsize",($^O=~/^MS/) ? "14" : "12");

  $prtFmtWidth	     =	val_or_def($confs,"prtfmtwidth",'595');
  $prtFmtHeight	     =	val_or_def($confs,"prtfmtheight",'842');
  $prtVMargin	     =	val_or_def($confs,"prtvmargin",'3c');
  $prtHMargin	     =	val_or_def($confs,"prthmargin",'2c');

  $psMedia	     =	val_or_def($confs,"psmedia",'%%DocumentMedia: A4 595 842 white()');
  $psFile=(exists $confs->{psfile}) ? tilde_expand($confs->{psfile}) : 'tred.ps';

  $maximizePrintSize  =	 val_or_def($confs,"maximizeprintsize",0);
  $createMacroMenu    =	 val_or_def($confs,"createmacromenu",0);
  $maxMenuLines	      =	 val_or_def($confs,"maxmenulines",20);
  $useCzechLocales    =	 val_or_def($confs,"useczechlocales",($^O !~ /^MS/));
  $Tk::strictMotif    =	 val_or_def($confs,"strictmotif",0);
  $printColors	      =	 val_or_def($confs,"printcolors",0);

  $gzip=val_or_def($confs,"gzip",(-x "/bin/gzip" ? "/bin/gzip -c" : undef));
  $zcat=val_or_def($confs,"zcat",(-x "/bin/zcat" ? "/bin/zcat" : $gzip));

  $ZBackend::gzip = $gzip;
  $ZBackend::zcat = $zcat;

  $cstsToFs = val_or_def($confs,"cststofs",undef);
  $fsToCsts = val_or_def($confs,"fstocsts",undef);

  $CSTSBackend::gzip = $gzip;
  $CSTSBackend::zcat = $zcat;
  $CSTSBackend::csts2fs=$cstsToFs;
  $CSTSBackend::fs2csts=$fsToCsts;

  $sgmls       = val_or_def($confs,"sgmls",undef);
  $sgmlsopts   = val_or_def($confs,"sgmlsopts",undef);
  $cstsdoctype = val_or_def($confs,"cstsdoctype","$libDir/csts.doctype");

  $CSTS_SGML_SP_Backend::gzip=$gzip;
  $CSTS_SGML_SP_Backend::zcat=$zcat;
  $CSTS_SGML_SP_Backend::sgmls=$sgmls;
  $CSTS_SGML_SP_Backend::sgmlsopts=$sgmlsopts;
  $CSTS_SGML_SP_Backend::doctype=$cstsdoctype;

  $keyboardDebug	      =	val_or_def($confs,"keyboarddebug",0);
  $hookDebug		      =	val_or_def($confs,"hookdebug",0);
  $macroDebug		      =	val_or_def($confs,"macrodebug",0);
  $tredDebug		      =	val_or_def($confs,"treddebug",0);
  $defaultTemplateMatchMethod =	val_or_def($confs,"searchmethod",'Exhaustive regular expression');
  $defaultMacroListOrder      =	val_or_def($confs,"macrolistorder",'M');
  $defCWidth		      =	val_or_def($confs,"canvaswidth",'18c');
  $defCHeight		      =	val_or_def($confs,"canvasheight",'12c');
  $geometry		      =	val_or_def($confs,"geometry",undef);
  $maxDisplayedValues	      =	val_or_def($confs,"maxdisplayedvalues",30);
  $maxDisplayedAttributes     =	val_or_def($confs,"maxdisplayedattributes",20);
  $lastAction		      =	val_or_def($confs,"lastaction",undef);

  &$set_user_config($confs) if (ref($set_user_config));
}

1;
